function image_segmentation(varargin)
% interactive image segmentation.
%
% Synopsis:
%  image_segmentation(path_to_image_file, kmeans_params)
%
% Description:
%  This function implements a simple interactive tool for image
%  segmentation. By using mouse a user can assign labels to 
%  class 1 (press left mouse button) or to 
%  class 2 (press right mouse button). 
%
%  By button SEGM calls the function gmm_segmentation. 
%  
%  BUTTON NAME      FUNCTION CALLED
%  "SEGM"        outLabels = gmm_segmentation(Image,inLabels,kmeans_params)
%
%  where
%   Image     [imHeight x imWidth x 3] is the input image.
%   inLabels  [imHeight x imWidth x 1] is the user labeling which can be a
%             number 0, 1, 2. The label zero means "no label assigned".
%   
%   outLabels [imHeight x imWidth] is the estimated labeling which should
%             countain only numbers 1 and 2. 
%
%  The estimated labeling (returned in the output argument outLables) is 
%  then displayed to a separate window.
%

switch varargin{1},
    case 'Dn', Dn;
    case 'Up', Up;
    case 'Plot', Plot;
    case 'Clear', Clear;
    case 'Segment-GMM', Segment('GMM');
    otherwise       
      Init(varargin{1}, varargin{2});
end

function Segment(method)

    hFig       = gcf;
    trn_labels = getappdata(gcf,'labels');
    img        = getappdata(gcf,'img');
    kmeans_params = getappdata(gcf, 'kmeans_params');
    
    if sum(trn_labels==1) ==0
        fprintf('No pixels labeled for class one (red)!\n');
        return;
    end
    if sum(trn_labels==2) ==0
        fprintf('No pixels labeled for class two (blue)!\n');
        return;
    end

    switch method
        case 'GMM'
            estim_labels = gmm_segmentation(img,trn_labels,kmeans_params);
    end        

    % display the estimated segmentation 
    idx1=find(estim_labels==1);
    idx2=find(estim_labels==2);

    M1 = zeros(size(img,1)*size(img,2),1);
    M2 = zeros(size(img,1)*size(img,2),1);
    M1(idx1) = 1;
    M2(idx2) = 1;
    M1 = reshape(M1,size(img,1),size(img,2));
    M2 = reshape(M2,size(img,1),size(img,2));
    
    img=double(img);
    I = img;
    t = 0.5;
    I(:,:,1) = 255*ones(size(M1)).*M1*t + (1-t)*img(:,:,1).*M1 + ...
               zeros(size(M2)).*M2*t + (1-t)*img(:,:,1).*M2;
    I(:,:,2) = zeros(size(M1)).*M1*t + (1-t)*img(:,:,2).*M1 + ...
               zeros(size(M2)).*M2*t + (1-t)*img(:,:,2).*M2;        
    I(:,:,3) = zeros(size(M1)).*M1*t + (1-t)*img(:,:,3).*M1 + ...
               255*ones(size(M2)).*M2*t + (1-t)*img(:,:,3).*M2;
           
    I = uint8(round(I));           
    hSegmentedImg = getappdata(hFig,'hSegmentedImg');
    if ~isempty(hSegmentedImg) && ~isempty(findobj('name','Estimated segmentation'))
       set(hSegmentedImg,'CData',I);
    else
        fig=figure;
        set(fig,'name','Estimated segmentation');
        hSegmentedImg = imshow(I,[]);
        setappdata(hFig,'hSegmentedImg',hSegmentedImg);
    end
    % imwrite(I,'segmented_img.png');

function Init(image_fname, kmeans_params)

    INIT_LAMBDA = 10;

    hFig = figure;  
    set(hFig, 'Units','normalized', ...
              'name',sprintf('Image file: %s', image_fname),...
              'WindowButtonDownFcn', ...
              'image_segmentation(''Dn'')');

    img = imread(image_fname);
    img_size = [size(img,1) size(img,2)];

    uicontrol( ...
            'Units','Normalized', ...
            'Callback','image_segmentation(''Segment-GMM'')',...
            'ListboxTop',0, ...
            'Position',[0.01 0.90-0.03 0.19 0.08], ...
            'String','SEGM');


    uicontrol( ...
            'Units','Normalized', ...
            'Callback','image_segmentation(''Clear'')',...
            'ListboxTop',0, ...
            'Position',[0.01 0.05 0.19 0.08], ...
            'String','Clear labels');

 
    hAxes1 = axes(...
           'Units','normalized', ...
           'Box','on', ...
           'XGrid','off', ...
           'YGrid','off', ...
           'Xlim',[1 img_size(2)],...
           'Ylim',[1 img_size(1)],...
           'NextPlot','add',...
           'drawmode', 'fast',...
           'interruptible', 'off',...
           'YDir','reverse',...
            'xlimmode', 'manual', ...
            'ylimmode', 'manual', ...
            'zlimmode', 'manual',...
            'DataAspectratio',[1 1 1],...
            'xtick',[],...
            'ytick',[],...
           'Position',[0.2 0.05 0.8 0.9]...
        );
    axis off;

    hImg = imagesc(img,'parent',hAxes1);
    hold on;
    drawnow;

    setappdata(hFig,'img',img);
    setappdata(hFig,'orig_cdata',get(hImg,'cdata'));
    setappdata(hFig,'hImg',hImg);
    setappdata(hFig,'image_fname', image_fname);
    setappdata(hFig,'img_size',img_size);
    setappdata(hFig,'labels',zeros(img_size,'uint8'));
    setappdata(hFig,'hSegmentedImg',[]);
    setappdata(hFig,'kmeans_params',kmeans_params);

function Clear()

    img_size = getappdata( gcf, 'img_size');
    orig_cdata = getappdata(gcf,'orig_cdata');
    setappdata(gcf,'labels',zeros(img_size,'uint8'));
    set(getappdata(gcf,'hImg'),'CData',orig_cdata);



function Up()

    set( gcf, 'WindowButtonMotionFcn', '' );
    set( gcf, 'WindowButtonUpFcn', '' );
        
function Dn()

    if strcmpi('normal',get(gcf, 'SelectionType'))
        setappdata(gcf,'label_type',1);
    else
        setappdata(gcf,'label_type',0);
    end

    set( gcf, 'WindowButtonMotionFcn', 'image_segmentation(''Plot'')' );
    set( gcf, 'WindowButtonUpFcn', 'image_segmentation(''Up'')' );

    Plot;

function Plot()
    img_size = getappdata( gcf, 'img_size');
    x =get( gca, 'currentpoint' );
    if( x(1) > 0 & x(1) < img_size(2) & x(3) > 0 & x(3) < img_size(1) );

       hImg = getappdata(gcf,'hImg');
       I = get(hImg,'CData');
       L = getappdata(gcf, 'labels');
       x1 = round(max(x(1)-5,1));
       x2 = round(min(x(1)+5,img_size(2)));
       y1 = round(max(x(3)-5,1));
       y2 = round(min(x(3)+5,img_size(1)));
       
       if getappdata( gcf,'label_type') == 1
           I(y1:y2,x1:x2,1) = 255;
           I(y1:y2,x1:x2,2) = 0;
           I(y1:y2,x1:x2,3) = 0;           
           L(y1:y2,x1:x2) = 1;
       else
           I(y1:y2,x1:x2,1) = 0;
           I(y1:y2,x1:x2,2) = 0;
           I(y1:y2,x1:x2,3) = 255;
           L(y1:y2,x1:x2) = 2;
       end
       set(hImg,'CData',I);
       setappdata(gcf,'labels',L);
    end
    
function Save()
    hFig       = gcf;
    hImg = getappdata(hFig,'hImg');
    trn_labels = getappdata(hFig,'labels');
    img = get(hImg,'CData');
    imwrite(img,'annotated_img1.png');
    